@@ -1,15 +1,19 @@
 package sentry
 
 import (
+	"bytes"
 	"context"
 	"encoding/json"
 	"errors"
 	"fmt"
+	"net"
+	"net/http"
 	"sync"
 	"sync/atomic"
 	"testing"
 	"time"
 
+	"github.com/getsentry/sentry-go/internal/debuglog"
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
 	pkgErrors "github.com/pkg/errors"
@@ -17,7 +21,7 @@ import (
 )
 
 func TestNewClientAllowsEmptyDSN(t *testing.T) {
-	transport := &TransportMock{}
+	transport := &MockTransport{}
 	client, err := NewClient(ClientOptions{
 		Transport: transport,
 	})
@@ -25,7 +29,7 @@ func TestNewClientAllowsEmptyDSN(t *testing.T) {
 		t.Fatalf("expected no error when creating client without a DNS but got %v", err)
 	}
 
-	client.CaptureException(errors.New("custom error"), nil, &ScopeMock{})
+	client.CaptureException(errors.New("custom error"), nil, &MockScope{})
 	assertEqual(t, transport.lastEvent.Exception[0].Value, "custom error")
 }
 
@@ -41,13 +45,14 @@ func (e customComplexError) AnswerToLife() string {
 	return "42"
 }
 
-func setupClientTest() (*Client, *ScopeMock, *TransportMock) {
-	scope := &ScopeMock{}
-	transport := &TransportMock{}
+func setupClientTest() (*Client, *MockScope, *MockTransport) {
+	scope := &MockScope{}
+	transport := &MockTransport{}
 	client, _ := NewClient(ClientOptions{
 		Dsn:       "http://whatever@example.com/1337",
 		Transport: transport,
-		Integrations: func(i []Integration) []Integration {
+		// keep default buffers enabled
+		Integrations: func(_ []Integration) []Integration {
 			return []Integration{}
 		},
 	})
@@ -162,11 +167,15 @@ func TestCaptureException(t *testing.T) {
 			err:  pkgErrors.WithStack(&customErr{}),
 			want: []Exception{
 				{
-					Type:  "*sentry.customErr",
-					Value: "wat",
+					Type:       "*sentry.customErr",
+					Value:      "wat",
+					Stacktrace: nil,
 					Mechanism: &Mechanism{
-						ExceptionID:      0,
-						IsExceptionGroup: true,
+						Type:             MechanismTypeChained,
+						ExceptionID:      1,
+						ParentID:         Pointer(0),
+						Source:           MechanismTypeUnwrap,
+						IsExceptionGroup: false,
 					},
 				},
 				{
@@ -174,9 +183,11 @@ func TestCaptureException(t *testing.T) {
 					Value:      "wat",
 					Stacktrace: &Stacktrace{Frames: []Frame{}},
 					Mechanism: &Mechanism{
-						ExceptionID:      1,
-						ParentID:         Pointer(0),
-						IsExceptionGroup: true,
+						Type:             MechanismTypeGeneric,
+						ExceptionID:      0,
+						ParentID:         nil,
+						Source:           "",
+						IsExceptionGroup: false,
 					},
 				},
 			},
@@ -197,11 +208,15 @@ func TestCaptureException(t *testing.T) {
 			err:  &customErrWithCause{cause: &customErr{}},
 			want: []Exception{
 				{
-					Type:  "*sentry.customErr",
-					Value: "wat",
+					Type:       "*sentry.customErr",
+					Value:      "wat",
+					Stacktrace: nil,
 					Mechanism: &Mechanism{
-						ExceptionID:      0,
-						IsExceptionGroup: true,
+						Type:             MechanismTypeChained,
+						ExceptionID:      1,
+						ParentID:         Pointer(0),
+						Source:           "cause",
+						IsExceptionGroup: false,
 					},
 				},
 				{
@@ -209,9 +224,11 @@ func TestCaptureException(t *testing.T) {
 					Value:      "err",
 					Stacktrace: &Stacktrace{Frames: []Frame{}},
 					Mechanism: &Mechanism{
-						ExceptionID:      1,
-						ParentID:         Pointer(0),
-						IsExceptionGroup: true,
+						Type:             MechanismTypeGeneric,
+						ExceptionID:      0,
+						ParentID:         nil,
+						Source:           "",
+						IsExceptionGroup: false,
 					},
 				},
 			},
@@ -221,11 +238,15 @@ func TestCaptureException(t *testing.T) {
 			err:  wrappedError{original: errors.New("original")},
 			want: []Exception{
 				{
-					Type:  "*errors.errorString",
-					Value: "original",
+					Type:       "*errors.errorString",
+					Value:      "original",
+					Stacktrace: nil,
 					Mechanism: &Mechanism{
-						ExceptionID:      0,
-						IsExceptionGroup: true,
+						Type:             MechanismTypeChained,
+						ExceptionID:      1,
+						ParentID:         Pointer(0),
+						Source:           MechanismTypeUnwrap,
+						IsExceptionGroup: false,
 					},
 				},
 				{
@@ -233,9 +254,11 @@ func TestCaptureException(t *testing.T) {
 					Value:      "wrapped: original",
 					Stacktrace: &Stacktrace{Frames: []Frame{}},
 					Mechanism: &Mechanism{
-						ExceptionID:      1,
-						ParentID:         Pointer(0),
-						IsExceptionGroup: true,
+						Type:             MechanismTypeGeneric,
+						ExceptionID:      0,
+						ParentID:         nil,
+						Source:           "",
+						IsExceptionGroup: false,
 					},
 				},
 			},
@@ -484,7 +507,7 @@ func TestApplyToScopeCanDropEvent(t *testing.T) {
 	client, scope, transport := setupClientTest()
 	scope.shouldDropEvent = true
 
-	client.AddEventProcessor(func(event *Event, hint *EventHint) *Event {
+	client.AddEventProcessor(func(event *Event, _ *EventHint) *Event {
 		if event == nil {
 			t.Errorf("EventProcessor received nil Event")
 		}
@@ -500,7 +523,7 @@ func TestApplyToScopeCanDropEvent(t *testing.T) {
 
 func TestBeforeSendCanDropEvent(t *testing.T) {
 	client, scope, transport := setupClientTest()
-	client.options.BeforeSend = func(event *Event, hint *EventHint) *Event {
+	client.options.BeforeSend = func(_ *Event, _ *EventHint) *Event {
 		return nil
 	}
 
@@ -527,16 +550,16 @@ func TestBeforeSendGetAccessToEventHint(t *testing.T) {
 }
 
 func TestBeforeSendTransactionCanDropTransaction(t *testing.T) {
-	transport := &TransportMock{}
+	transport := &MockTransport{}
 	ctx := NewTestContext(ClientOptions{
 		EnableTracing:    true,
 		TracesSampleRate: 1.0,
 		Transport:        transport,
-		BeforeSend: func(event *Event, hint *EventHint) *Event {
+		BeforeSend: func(event *Event, _ *EventHint) *Event {
 			t.Error("beforeSend should not be called")
 			return event
 		},
-		BeforeSendTransaction: func(event *Event, hint *EventHint) *Event {
+		BeforeSendTransaction: func(event *Event, _ *EventHint) *Event {
 			assertEqual(t, event.Transaction, "Foo")
 			return nil
 		},
@@ -553,16 +576,16 @@ func TestBeforeSendTransactionCanDropTransaction(t *testing.T) {
 }
 
 func TestBeforeSendTransactionIsCalled(t *testing.T) {
-	transport := &TransportMock{}
+	transport := &MockTransport{}
 	ctx := NewTestContext(ClientOptions{
 		EnableTracing:    true,
 		TracesSampleRate: 1.0,
 		Transport:        transport,
-		BeforeSend: func(event *Event, hint *EventHint) *Event {
+		BeforeSend: func(event *Event, _ *EventHint) *Event {
 			t.Error("beforeSend should not be called")
 			return event
 		},
-		BeforeSendTransaction: func(event *Event, hint *EventHint) *Event {
+		BeforeSendTransaction: func(event *Event, _ *EventHint) *Event {
 			assertEqual(t, event.Transaction, "Foo")
 			event.Transaction = "Bar"
 			return event
@@ -615,8 +638,8 @@ func TestIgnoreErrors(t *testing.T) {
 
 	for name, tt := range tests {
 		t.Run(name, func(t *testing.T) {
-			scope := &ScopeMock{}
-			transport := &TransportMock{}
+			scope := &MockScope{}
+			transport := &MockTransport{}
 			client, err := NewClient(ClientOptions{
 				Transport:    transport,
 				IgnoreErrors: tt.ignoreErrors,
@@ -628,7 +651,7 @@ func TestIgnoreErrors(t *testing.T) {
 			client.CaptureMessage(tt.message, nil, scope)
 
 			dropped := transport.lastEvent == nil
-			if !(tt.expectDrop == dropped) {
+			if tt.expectDrop != dropped {
 				t.Errorf("expected event to be dropped")
 			}
 		})
@@ -670,7 +693,7 @@ func TestIgnoreTransactions(t *testing.T) {
 
 	for name, tt := range tests {
 		t.Run(name, func(t *testing.T) {
-			transport := &TransportMock{}
+			transport := &MockTransport{}
 			ctx := NewTestContext(ClientOptions{
 				EnableTracing:      true,
 				TracesSampleRate:   1.0,
@@ -684,13 +707,144 @@ func TestIgnoreTransactions(t *testing.T) {
 			transaction.Finish()
 
 			dropped := transport.lastEvent == nil
-			if !(tt.expectDrop == dropped) {
+			if tt.expectDrop != dropped {
 				t.Errorf("expected event to be dropped")
 			}
 		})
 	}
 }
 
+func TestTraceIgnoreStatusCode_EmptyCode(t *testing.T) {
+	transport := &MockTransport{}
+	ctx := NewTestContext(ClientOptions{
+		EnableTracing:    true,
+		TracesSampleRate: 1.0,
+		Transport:        transport,
+	})
+
+	transaction := StartTransaction(ctx, "test")
+	// Transaction has no http.response.status_code
+	transaction.Finish()
+
+	dropped := transport.lastEvent == nil
+	assertEqual(t, dropped, false, "expected transaction to not be dropped")
+}
+
+func TestTraceIgnoreStatusCodes(t *testing.T) {
+	tests := map[string]struct {
+		ignoreStatusCodes [][]int
+		statusCode        interface{}
+		expectDrop        bool
+	}{
+		"Default behavior: ignoreStatusCodes = nil, should drop 404s": {
+			statusCode:        404,
+			ignoreStatusCodes: nil,
+			expectDrop:        true,
+		},
+		"Specify No ignored codes": {
+			statusCode:        404,
+			ignoreStatusCodes: [][]int{},
+			expectDrop:        false,
+		},
+		"Status code not in ignore ranges": {
+			statusCode:        500,
+			ignoreStatusCodes: [][]int{{400, 405}},
+			expectDrop:        false,
+		},
+		"404 in ignore range": {
+			statusCode:        404,
+			ignoreStatusCodes: [][]int{{400, 405}},
+			expectDrop:        true,
+		},
+		"403 in ignore range": {
+			statusCode:        403,
+			ignoreStatusCodes: [][]int{{400, 405}},
+			expectDrop:        true,
+		},
+		"200 not ignored": {
+			statusCode:        200,
+			ignoreStatusCodes: [][]int{{400, 405}},
+			expectDrop:        false,
+		},
+		"wrong code not ignored": {
+			statusCode:        "something",
+			ignoreStatusCodes: [][]int{{400, 405}},
+			expectDrop:        false,
+		},
+		"Single status code as single-element slice": {
+			statusCode:        404,
+			ignoreStatusCodes: [][]int{{404}},
+			expectDrop:        true,
+		},
+		"Single status code not in single-element slice": {
+			statusCode:        500,
+			ignoreStatusCodes: [][]int{{404}},
+			expectDrop:        false,
+		},
+		"Multiple single codes": {
+			statusCode:        500,
+			ignoreStatusCodes: [][]int{{404}, {500}},
+			expectDrop:        true,
+		},
+		"Multiple ranges - code in first range": {
+			statusCode:        404,
+			ignoreStatusCodes: [][]int{{400, 405}, {500, 599}},
+			expectDrop:        true,
+		},
+		"Multiple ranges - code in second range": {
+			statusCode:        500,
+			ignoreStatusCodes: [][]int{{400, 405}, {500, 599}},
+			expectDrop:        true,
+		},
+		"Multiple ranges - code not in any range": {
+			statusCode:        200,
+			ignoreStatusCodes: [][]int{{400, 405}, {500, 599}},
+			expectDrop:        false,
+		},
+		"Mixed single codes and ranges": {
+			statusCode:        404,
+			ignoreStatusCodes: [][]int{{404}, {500, 599}},
+			expectDrop:        true,
+		},
+		"Mixed single codes and ranges - code in range": {
+			statusCode:        500,
+			ignoreStatusCodes: [][]int{{404}, {500, 599}},
+			expectDrop:        true,
+		},
+		"Mixed single codes and ranges - code not matched": {
+			statusCode:        200,
+			ignoreStatusCodes: [][]int{{404}, {500, 599}},
+			expectDrop:        false,
+		},
+	}
+
+	for name, tt := range tests {
+		t.Run(name, func(t *testing.T) {
+			transport := &MockTransport{}
+			ctx := NewTestContext(ClientOptions{
+				EnableTracing:          true,
+				TracesSampleRate:       1.0,
+				Transport:              transport,
+				TraceIgnoreStatusCodes: tt.ignoreStatusCodes,
+			})
+
+			transaction := StartTransaction(ctx, "test")
+			// Simulate HTTP response data like the integrations do
+			transaction.SetData("http.response.status_code", tt.statusCode)
+			transaction.Finish()
+
+			dropped := transport.lastEvent == nil
+			if tt.expectDrop != dropped {
+				if tt.expectDrop {
+					t.Errorf("expected transaction with status code %d to be dropped", tt.statusCode)
+				} else {
+					t.Errorf("expected transaction with status code %d not to be dropped", tt.statusCode)
+				}
+			}
+		})
+	}
+}
+
 func TestSampleRate(t *testing.T) {
 	tests := []struct {
 		SampleRate float64
@@ -749,7 +903,7 @@ func TestSampleRate(t *testing.T) {
 func BenchmarkProcessEvent(b *testing.B) {
 	c, err := NewClient(ClientOptions{
 		SampleRate: 0.25,
-		Transport:  &TransportMock{},
+		Transport:  &MockTransport{},
 	})
 	if err != nil {
 		b.Fatal(err)
@@ -850,7 +1004,7 @@ func TestCustomMaxSpansProperty(t *testing.T) {
 
 	properClient, _ := NewClient(ClientOptions{
 		MaxSpans:  3000,
-		Transport: &TransportMock{},
+		Transport: &MockTransport{},
 	})
 
 	assertEqual(t, properClient.Options().MaxSpans, 3000)
@@ -865,9 +1019,59 @@ func TestSDKIdentifier(t *testing.T) {
 }
 
 func TestClientSetsUpTransport(t *testing.T) {
-	client, _ := NewClient(ClientOptions{Dsn: testDsn})
-	require.IsType(t, &HTTPTransport{}, client.Transport)
+	client, _ := NewClient(ClientOptions{
+		Dsn: testDsn,
+		HTTPClient: &http.Client{
+			Transport: &http.Transport{
+				DialContext: func(_ context.Context, _, _ string) (net.Conn, error) {
+					return nil, fmt.Errorf("mock transport - no real connections")
+				},
+			},
+		},
+		Transport: &MockTransport{},
+	})
+	require.IsType(t, &MockTransport{}, client.Transport)
 
 	client, _ = NewClient(ClientOptions{})
 	require.IsType(t, &noopTransport{}, client.Transport)
 }
+
+// func TestClient_SetupTelemetryBuffer_WithDSN(t *testing.T) {
+//	client, err := NewClient(ClientOptions{
+//		Dsn: "https://public@localhost/1",
+//	})
+//	if err != nil {
+//		t.Fatalf("unexpected error: %v", err)
+//	}
+//
+//	if client.telemetryBuffer == nil {
+//		t.Fatal("expected telemetryBuffer to be initialized")
+//	}
+//
+//	if _, ok := client.Transport.(*internalAsyncTransportAdapter); !ok {
+//		t.Fatalf("expected internalAsyncTransportAdapter, got %T", client.Transport)
+//	}
+//
+//	if !client.telemetryBuffer.Add(NewEvent()) {
+//		t.Fatal("expected Add to succeed with default buffers")
+//	}
+//}
+
+func TestClient_SetupTelemetryBuffer_NoDSN(t *testing.T) {
+	var buf bytes.Buffer
+	debuglog.SetOutput(&buf)
+	defer debuglog.SetOutput(&bytes.Buffer{})
+
+	client, err := NewClient(ClientOptions{})
+	if err != nil {
+		t.Fatalf("unexpected error: %v", err)
+	}
+
+	if client.telemetryBuffer != nil {
+		t.Fatal("expected telemetryBuffer to be nil when DSN is missing")
+	}
+
+	if _, ok := client.Transport.(*noopTransport); !ok {
+		t.Fatalf("expected noopTransport, got %T", client.Transport)
+	}
+}
