@@ -9,7 +9,7 @@ import (
 	"github.com/getsentry/sentry-go"
 )
 
-func TestConcurrentScopeUsage(t *testing.T) {
+func TestConcurrentScopeUsage(_ *testing.T) {
 	var wg sync.WaitGroup
 
 	for i := 0; i < 10; i++ {
@@ -57,6 +57,8 @@ func touchScope(scope *sentry.Scope, x int) {
 	scope.AddAttachment(&sentry.Attachment{Filename: "foo.txt"})
 	scope.SetUser(sentry.User{ID: "foo"})
 	scope.SetRequest(httptest.NewRequest("GET", "/foo", nil))
+	scope.SetPropagationContext(sentry.NewPropagationContext())
+	scope.SetSpan(&sentry.Span{TraceID: sentry.TraceIDFromHex("d49d9bf66f13450b81f65bc51cf49c03")})
 
 	sentry.CaptureException(fmt.Errorf("error %d", x))
 
