@@ -6,6 +6,7 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
+	"net"
 	"net/http"
 	"net/http/httptest"
 	"net/http/httptrace"
@@ -17,6 +18,7 @@ import (
 
 	"github.com/getsentry/sentry-go/internal/testutils"
 	"github.com/google/go-cmp/cmp"
+	"go.uber.org/goleak"
 )
 
 type unserializableType struct {
@@ -229,59 +231,6 @@ func TestEnvelopeFromEventWithAttachments(t *testing.T) {
 	}
 }
 
-func TestEnvelopeFromTransactionWithProfile(t *testing.T) {
-	event := newTestEvent(transactionType)
-	event.sdkMetaData.transactionProfile = &profileInfo{
-		Trace: &profileTrace{
-			Frames: []*Frame{
-				{
-					Function: "func",
-					Module:   "module",
-					Filename: "file.go",
-					Lineno:   42,
-					Colno:    24,
-				},
-			},
-			Samples: []profileSample{
-				{
-					ElapsedSinceStartNS: 10,
-					StackID:             2,
-					ThreadID:            3,
-				},
-			},
-			Stacks: []profileStack{{0}},
-			ThreadMetadata: map[uint64]*profileThreadMetadata{
-				1: {Name: "GO 1"},
-			},
-		},
-		Transaction: profileTransaction{
-			ActiveThreadID: 1,
-			DurationNS:     2,
-			ID:             "3",
-			Name:           "tx-name",
-			TraceID:        "trace-id",
-		},
-	}
-	sentAt := time.Unix(0, 0).UTC()
-
-	body := json.RawMessage(`{"type":"transaction","fields":"omitted"}`)
-
-	b, err := envelopeFromBody(event, newTestDSN(t), sentAt, body)
-	if err != nil {
-		t.Fatal(err)
-	}
-	got := b.String()
-	want := `{"event_id":"b81c5be4d31e48959103a1f878a1efcb","sent_at":"1970-01-01T00:00:00Z","dsn":"http://public@example.com/sentry/1","sdk":{"name":"sentry.go","version":"0.0.1"}}
-{"type":"transaction","length":41}
-{"type":"transaction","fields":"omitted"}
-{"type":"profile","length":618}
-{"device":{"architecture":"","classification":"","locale":"","manufacturer":"","model":""},"event_id":"","os":{"build_number":"","name":"","version":""},"platform":"","release":"","dist":"","runtime":{"name":"","version":""},"timestamp":"0001-01-01T00:00:00Z","profile":{"frames":[{"function":"func","module":"module","filename":"file.go","lineno":42,"colno":24,"in_app":false}],"samples":[{"elapsed_since_start_ns":10,"stack_id":2,"thread_id":3}],"stacks":[[0]],"thread_metadata":{"1":{"name":"GO 1"}}},"transaction":{"active_thread_id":1,"duration_ns":2,"id":"3","name":"tx-name","trace_id":"trace-id"},"version":""}
-`
-	if diff := cmp.Diff(want, got); diff != "" {
-		t.Errorf("Envelope mismatch (-want +got):\n%s", diff)
-	}
-}
-
 func TestEnvelopeFromCheckInEvent(t *testing.T) {
 	event := newTestEvent(checkInType)
 	event.CheckIn = &CheckIn{
@@ -312,6 +261,44 @@ func TestEnvelopeFromCheckInEvent(t *testing.T) {
 	}
 }
 
+func TestEnvelopeFromLogEvent(t *testing.T) {
+	event := newTestEvent(logEvent.Type)
+	event.Logs = []Log{
+		{
+			TraceID:  TraceIDFromHex(LogTraceID),
+			Level:    LogLevelInfo,
+			Severity: LogSeverityInfo,
+			Body:     "test log message",
+			Attributes: map[string]Attribute{
+				"sentry.release":        {Value: "v1.2.3", Type: "string"},
+				"sentry.environment":    {Value: "testing", Type: "string"},
+				"sentry.server.address": {Value: "test-server", Type: "string"},
+				"sentry.sdk.name":       {Value: "sentry.go", Type: "string"},
+				"sentry.sdk.version":    {Value: "0.0.1", Type: "string"},
+				"key.int":               {Value: 42, Type: "integer"},
+				"key.string":            {Value: "str", Type: "string"},
+				"key.float":             {Value: 42.2, Type: "double"},
+				"key.bool":              {Value: true, Type: "boolean"},
+			},
+		},
+	}
+	sentAt := time.Unix(0, 0).UTC()
+
+	body := getRequestBodyFromEvent(event)
+	b, err := envelopeFromBody(event, newTestDSN(t), sentAt, body)
+	if err != nil {
+		t.Fatal(err)
+	}
+	got := b.String()
+	want := `{"event_id":"b81c5be4d31e48959103a1f878a1efcb","sent_at":"1970-01-01T00:00:00Z","dsn":"http://public@example.com/sentry/1","sdk":{"name":"sentry.go","version":"0.0.1"}}
+{"type":"log","item_count":1,"content_type":"application/vnd.sentry.items.log+json"}
+{"event_id":"b81c5be4d31e48959103a1f878a1efcb","sdk":{"name":"sentry.go","version":"0.0.1"},"user":{},"items":[{"timestamp":"0001-01-01T00:00:00Z","trace_id":"d49d9bf66f13450b81f65bc51cf49c03","level":"info","severity_number":9,"body":"test log message","attributes":{"key.bool":{"value":true,"type":"boolean"},"key.float":{"value":42.2,"type":"double"},"key.int":{"value":42,"type":"integer"},"key.string":{"value":"str","type":"string"},"sentry.environment":{"value":"testing","type":"string"},"sentry.release":{"value":"v1.2.3","type":"string"},"sentry.sdk.name":{"value":"sentry.go","type":"string"},"sentry.sdk.version":{"value":"0.0.1","type":"string"},"sentry.server.address":{"value":"test-server","type":"string"}}}]}
+`
+	if diff := cmp.Diff(want, got); diff != "" {
+		t.Errorf("Envelope mismatch (-want +got):\n%s", diff)
+	}
+}
+
 func TestGetRequestFromEvent(t *testing.T) {
 	testCases := []struct {
 		testName string
@@ -385,7 +372,7 @@ type testHTTPServer struct {
 func newTestHTTPServer(t *testing.T) *testHTTPServer {
 	ch := make(chan bool)
 	eventCounter := new(uint64)
-	handler := func(w http.ResponseWriter, r *http.Request) {
+	handler := func(_ http.ResponseWriter, r *http.Request) {
 		var event struct {
 			EventID string `json:"event_id"`
 		}
@@ -503,6 +490,120 @@ func TestHTTPTransport(t *testing.T) {
 		wg.Wait()
 	})
 }
+func TestHTTPTransport_CloseMultipleTimes(t *testing.T) {
+	server := newTestHTTPServer(t)
+	defer server.Close()
+	transport := NewHTTPTransport()
+	transport.Configure(ClientOptions{
+		Dsn:        fmt.Sprintf("https://test@%s/1", server.Listener.Addr()),
+		HTTPClient: server.Client(),
+	})
+
+	// Closing multiple times should not panic.
+	for i := 0; i < 10; i++ {
+		transport.Close()
+	}
+
+	// Verify the done channel is closed
+	select {
+	case <-transport.done:
+		// Expected - channel should be closed
+	case <-time.After(time.Second):
+		t.Fatal("transport.done not closed")
+	}
+}
+
+func TestHTTPTransport_FlushWithContext(t *testing.T) {
+	server := newTestHTTPServer(t)
+	defer server.Close()
+
+	transport := NewHTTPTransport()
+	transport.Configure(ClientOptions{
+		Dsn:        fmt.Sprintf("https://test@%s/1", server.Listener.Addr()),
+		HTTPClient: server.Client(),
+	})
+
+	// Helpers
+
+	transportSendTestEvent := func(t *testing.T) (id string) {
+		t.Helper()
+
+		e := NewEvent()
+		id = uuid()
+		e.EventID = EventID(id)
+
+		transport.SendEvent(e)
+
+		t.Logf("[CLIENT] {%.4s} event sent", e.EventID)
+		return id
+	}
+
+	transportMustFlushWithCtx := func(t *testing.T, id string) {
+		t.Helper()
+		cancelCtx, cancel := context.WithCancel(context.Background())
+		defer cancel()
+		ok := transport.FlushWithContext(cancelCtx)
+		if !ok {
+			t.Fatalf("[CLIENT] {%.4s} Flush() timed out", id)
+		}
+	}
+
+	serverEventCountMustBe := func(t *testing.T, n uint64) {
+		t.Helper()
+
+		count := server.EventCount()
+		if count != n {
+			t.Fatalf("[SERVER] event count = %d, want %d", count, n)
+		}
+	}
+
+	testSendSingleEvent := func(t *testing.T) {
+		// Sending a single event should increase the server event count by
+		// exactly one.
+
+		initialCount := server.EventCount()
+		id := transportSendTestEvent(t)
+
+		// Server is blocked waiting for us, right now count must not have
+		// changed yet.
+		serverEventCountMustBe(t, initialCount)
+
+		// After unblocking the server, Flush must guarantee that the server
+		// event count increased by one.
+		server.Unblock()
+		transportMustFlushWithCtx(t, id)
+		serverEventCountMustBe(t, initialCount+1)
+	}
+
+	// Actual tests
+	t.Run("SendSingleEvent", testSendSingleEvent)
+
+	t.Run("FlushMultipleTimes", func(t *testing.T) {
+		// Flushing multiple times should not increase the server event count.
+
+		initialCount := server.EventCount()
+		for i := 0; i < 10; i++ {
+			transportMustFlushWithCtx(t, fmt.Sprintf("loop%d", i))
+		}
+		serverEventCountMustBe(t, initialCount)
+	})
+
+	t.Run("ConcurrentSendAndFlush", func(t *testing.T) {
+		// It should be safe to send events and flush concurrently.
+
+		var wg sync.WaitGroup
+		wg.Add(2)
+		go func() {
+			defer wg.Done()
+			testSendSingleEvent(t)
+		}()
+		go func() {
+			defer wg.Done()
+			transportMustFlushWithCtx(t, "from goroutine")
+		}()
+		wg.Wait()
+	})
+}
 
 // httptraceRoundTripper implements http.RoundTripper by wrapping
 // http.DefaultTransport and keeps track of whether TCP connections have been
@@ -533,7 +634,7 @@ func testKeepAlive(t *testing.T, tr Transport) {
 	// unexpectedly large response from Relay -- the SDK should not try to
 	// consume arbitrarily large responses.
 	largeResponse := false
-	srv := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+	srv := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 		// Simulates a response from Relay. The event_id is arbitrary,
 		// it doesn't matter for this test.
 		fmt.Fprintln(w, `{"id":"ec71d87189164e79ab1e61030c183af0"}`)
@@ -683,33 +784,113 @@ func testRateLimiting(t *testing.T, tr Transport) {
 	}
 }
 
-func TestEnvelopeFromMetricBody(t *testing.T) {
-	event := newTestEvent(metricType)
-	event.Metrics = append(event.Metrics,
-		NewCounterMetric("counter", Second(), map[string]string{"foo": "bar", "route": "GET /foo"}, 1597790835, 1.0),
-		NewDistributionMetric("distribution", Second(), map[string]string{"$foo$": "%bar%"}, 1597790835, 1.0),
-		NewGaugeMetric("gauge", Second(), map[string]string{"föö": "bär"}, 1597790835, 1.0),
-		NewSetMetric[int]("set", Second(), map[string]string{"%{key}": "$value$"}, 1597790835, 1),
-		NewCounterMetric("no_tags", Second(), nil, 1597790835, 1.0),
-	)
-	sentAt := time.Unix(0, 0).UTC()
+func TestHTTPTransportDoesntLeakGoroutines(t *testing.T) {
+	defer goleak.VerifyNone(t, goleak.IgnoreCurrent())
 
-	body := getRequestBodyFromEvent(event)
+	transport := NewHTTPTransport()
+	transport.Configure(ClientOptions{
+		Dsn: "https://test@foobar/1",
+		HTTPClient: &http.Client{
+			Transport: &http.Transport{
+				DialContext: func(_ context.Context, _, _ string) (net.Conn, error) {
+					return nil, fmt.Errorf("mock transport - no real connections")
+				},
+			},
+		},
+	})
 
-	b, err := envelopeFromBody(event, newTestDSN(t), sentAt, body)
-	if err != nil {
-		t.Fatal(err)
-	}
-	got := b.String()
-	want := `{"event_id":"b81c5be4d31e48959103a1f878a1efcb","sent_at":"1970-01-01T00:00:00Z","dsn":"http://public@example.com/sentry/1","sdk":{"name":"sentry.go","version":"0.0.1"}}
-{"type":"statsd","length":218}
-counter@second:1|c|#foo:bar,route:GET /foo|T1597790835
-distribution@second:1|d|#_foo_:bar|T1597790835
-gauge@second:1:1:1:1:1|g|#f_:br|T1597790835
-set@second:1|s|#_key_:$value$|T1597790835
-no_tags@second:1|c|T1597790835
-`
-	if diff := cmp.Diff(want, got); diff != "" {
-		t.Errorf("Envelope mismatch (-want +got):\n%s", diff)
+	transport.Flush(testutils.FlushTimeout())
+	transport.Close()
+}
+
+func TestHTTPTransportClose(t *testing.T) {
+	transport := NewHTTPTransport()
+	transport.Configure(ClientOptions{
+		Dsn: "https://test@foobar/1",
+		HTTPClient: &http.Client{
+			Transport: &http.Transport{
+				DialContext: func(_ context.Context, _, _ string) (net.Conn, error) {
+					return nil, fmt.Errorf("mock transport - no real connections")
+				},
+			},
+		},
+	})
+
+	transport.Close()
+
+	select {
+	case <-transport.done:
+	case <-time.After(time.Second):
+		t.Error("transport.done not closed after Close")
 	}
 }
+
+func TestHTTPSyncTransportClose(_ *testing.T) {
+	// Close does not do anything for HTTPSyncTransport, added for coverage.
+	transport := HTTPSyncTransport{}
+	transport.Close()
+
+	tr := noopTransport{}
+	tr.Close()
+}
+
+func TestHTTPSyncTransport_Flush(t *testing.T) {
+	// Flush does not do anything for HTTPSyncTransport, added for coverage.
+	transport := HTTPSyncTransport{}
+	transport.Flush(testutils.FlushTimeout())
+
+	tr := noopTransport{}
+	ret := tr.Flush(testutils.FlushTimeout())
+	if ret != true {
+		t.Fatalf("expected Flush to be true, got: %v", ret)
+	}
+}
+
+func TestHTTPSyncTransport_FlushWithContext(_ *testing.T) {
+	// FlushWithContext does not do anything for HTTPSyncTransport, added for coverage.
+	transport := HTTPSyncTransport{}
+	cancelCtx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+	transport.FlushWithContext(cancelCtx)
+
+	tr := noopTransport{}
+	tr.FlushWithContext(cancelCtx)
+}
+
+func TestInternalAsyncTransportAdapter(t *testing.T) {
+	transport := newInternalAsyncTransport()
+
+	transport.Configure(ClientOptions{
+		Dsn: "",
+	})
+
+	event := NewEvent()
+	event.Message = "test message"
+	transport.SendEvent(event)
+
+	if !transport.Flush(time.Second) {
+		t.Error("Flush should return true")
+	}
+
+	if !transport.FlushWithContext(context.Background()) {
+		t.Error("FlushWithContext should return true")
+	}
+
+	transport.Close()
+}
+
+func TestInternalAsyncTransportAdapter_WithValidDSN(_ *testing.T) {
+	transport := newInternalAsyncTransport()
+
+	transport.Configure(ClientOptions{
+		Dsn: "https://public@example.com/1",
+	})
+
+	event := NewEvent()
+	event.Message = "test message"
+	transport.SendEvent(event)
+
+	transport.Flush(100 * time.Millisecond)
+
+	transport.Close()
+}
